import base64
import hashlib

"""
This is the only exception when we use the standard XML API since
defusedxml does not seem to support canonicalization.

Albeit a catch-all library, lxml is not an option since it is a binary.
QGIS and QGIS plugin publishing chaintools prohibit including bins: https://plugins.qgis.org/publish/
"""
from ...common.xml import XmlNode, findXmlSubnode, canonicalizeAndSerializeXml


# https://eagri.cz/public/web/mze/farmar/elektronicka-vymena-dat/postup-tvorby-hashe/
def computeAuthenticationToken(request_without_auth_token: XmlNode, hashed_ws_key: str):
    wskey_hash_bin = bytes.fromhex(hashed_ws_key)
    request_body_bin = canonicalizeAndSerializeXml(findXmlSubnode(request_without_auth_token, 'Body'))
    ws_key_and_body_bin = request_body_bin + wskey_hash_bin
    final_hash_bin = hashlib.sha512(ws_key_and_body_bin).digest()
    return base64.b64encode(final_hash_bin).decode()
